SystemOrganization addCategory: #'SomethingIt-Haskell-Process'!
SystemOrganization addCategory: #'SomethingIt-Haskell-Test'!

SIExternalInterpreter subclass: #SIHaskell
	instanceVariableNames: 'programs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-Haskell-Process'!

!SIHaskell class methodsFor: 'examples' stamp: 'tomooda 10/30/2012 13:26'!
exampleCallback
	"SIHaskell exampleCallback inspect"
	| ghci at |
	ghci := SIHaskell prelude.
	ghci program: 'let at array index = messageSend array "at:" [index] :: IO Int'.
	at := 'at' asHaskellExpressionTyped: '[Int]->Int->IO Int' in: ghci.
	^ (1 to: 3) collect: [ :index | at value: #(4 4 1) asOrderedCollection value: index]! !

!SIHaskell class methodsFor: 'examples' stamp: 'tomooda 10/30/2012 14:06'!
exampleCallback2
	"SIHaskell exampleCallback2 inspect"
	| ghci at |
	ghci := SIHaskell source: '
import SomethingIt (messageSend)
at array index = messageSend array "at:" [index] :: IO Int
'.
	at := 'at' asHaskellExpressionTyped: '[Int]->Int->IO Int' in: ghci.
	^ (1 to: 3) collect: [ :index | at value: #(4 4 1) asOrderedCollection value: index]! !

!SIHaskell class methodsFor: 'examples' stamp: 'tomooda 11/1/2012 11:51'!
exampleGravity
	"SIHaskell exampleGravity"
	| gravity form morph window colors |
	form := Form extent: 500 @ 500 depth: 32.
	morph := form asMorph.
	window := morph openInWindowLabeled: 'Gravity by Haskell'.
	colors := (1 to: 3)
				collect: [:i | Color
						h: i * 360.0 / 10
						s: 1.0
						v: 1.0].
	[| haskell points velocities add |
	haskell := SIHaskell source: '
data Point = Point Double Double deriving (Read, Show)
plus :: Point -> Point -> Point
plus (Point x1 y1) (Point x2 y2) = Point (x1+x2) (y1+y2)

minus :: Point -> Point -> Point
minus (Point x1 y1) (Point x2 y2) = Point (x1-x2) (y1-y2)

neg :: Point -> Point
neg (Point x y) = Point (-x) (-y)

dist2 :: Point -> Double
dist2 (Point x y) = x**2 + y**2

scale :: Double -> Point -> Point
scale a (Point x y) = Point (a * x) (a * y)

gravity :: [Point] -> [Point]
gravity ps = [total [gravity1 p1 p2 | p2 <- ps] | p1 <- ps]
     where
     total = foldr1 plus
     gravity1 p1 p2
          | d < 400.0 = Point 0.0 0.0
          | otherwise = scale (gmn / d / sqrt d) v
               where
               v = minus p2 p1
               d = dist2 v
               gmn = 50.0
'.
	haskell printer
		addClassMap: #Point
		to: [:point :haskellStream | 
			haskellStream nextPutAll: 'Point ('.
			point x printOn: haskellStream.
			haskellStream nextPutAll: ') ('.
			point y printOn: haskellStream.
			haskellStream nextPut: $)].
	haskell parser
		addAlgebraicDataMap: 'Point'
		to: [:constructor :args | args first @ args second].
	gravity := 'gravity' asHaskellExpressionIn: haskell.
	points := {100.0 @ 250.0. 250.0 @ 250.0. 250.0 @ 350.0} asOrderedCollection.
	velocities := {0.0 @ 0.0. 0.0 @ 0.0. 0.0 @ 0.0} asOrderedCollection.
	add := [:array1 :array2 | 1
				to: array1 size
				do: [:index | array1 at: index put: (array1 at: index)
							+ (array2 at: index)]].
	2000
		timesRepeat: [| forces canvas |
			forces := gravity value: points.
			add value: velocities value: forces.
			add value: points value: velocities.
			canvas := form getCanvas.
			canvas fillColor: Color white.
			points
				with: colors
				do: [:point :color | canvas
						fillOval: (point - 4 extent: 8 @ 8)
						color: color].
			canvas flush.
			morph changed.
			(Delay forMilliseconds: 10) wait].
	morph delete.
	window delete]
		forkAt: Processor userBackgroundPriority! !

!SIHaskell class methodsFor: 'examples' stamp: 'TomohiroOda 6/7/2012 20:10'!
exampleLikeAndDislike
	"SIHaskell exampleLikeAndDislike inspect"

	| haskell output |
	haskell := SIHaskell prelude.
	haskell program: 'data Faddiness a = Like a | Dislike a deriving Show'.
	haskell program: 'let fancy x = if even x then Like x else Dislike x'.
	output := Array with: (haskell evaluate: 'fancy 1') with: (haskell evaluate: 'fancy 1024') with: (haskell evaluate: 'fancy 13').
	haskell close.
	^ output! !

!SIHaskell class methodsFor: 'examples' stamp: 'TomohiroOda 9/11/2012 09:33'!
exampleMath
	"SIHaskell exampleMath inspect"

	| haskell |
	haskell := SIHaskell prelude.
	^ [ haskell evaluate: '1 + 2 * 3' ] ensure: [ haskell close ]! !

!SIHaskell class methodsFor: 'examples' stamp: 'TO 9/20/2012 14:07'!
exampleTypeError
	"SIHaskell exampleTypeError"
	| haskell |
	haskell := SIHaskell prelude.
	^ [haskell evaluate: '1 + "1"']
		ensure: [haskell close]! !

!SIHaskell class methodsFor: 'accessing' stamp: 'TO 10/19/2012 11:30'!
ghciPath
	^ '/usr/bin/ghci'! !

!SIHaskell class methodsFor: 'class initialization' stamp: 'tomooda 10/30/2012 14:02'!
initialize
	"SIHaskell initialize"
	super initialize.
	FileStream
		forceNewFileNamed: 'SomethingIt.hs'
		do: [:stream | 
			stream ascii.
			stream lineEndConvention: #lf.
			stream nextPutAll: 'module SomethingIt (SIObject(SIObject), messageSend) where
data SIObject = SIObject Int deriving Show
data SIMessageSend receiver args = SIMessageSend receiver String args deriving Show
messageSend :: (Show r, Show a, Read ret) => r -> String -> a -> IO ret
messageSend r s a = do
    putChar ''!!'';
    putStrLn $ show (SIMessageSend r s a);
    l <- getLine;
    return $ read l
'.
			stream close]! !

!SIHaskell class methodsFor: 'instance creation' stamp: 'TO 7/26/2012 13:38'!
load: aFileNameString 
	^ self prelude load: aFileNameString;
		 yourself! !

!SIHaskell class methodsFor: 'instance creation' stamp: 'TomohiroOda 6/7/2012 13:46'!
prelude
	^ self new! !

!SIHaskell class methodsFor: 'instance creation' stamp: 'tomooda 10/30/2012 13:47'!
source: aString 
	^ SIFileUtils
		temporaryFile: 'SIHaskell-source:-'
		randomSuffixSize: 8
		suffix: '.hs'
		do: [:name :stream | 
			stream ascii.
			stream lineEndConvention: #lf.
			stream nextPutAll: aString.
			stream close.
			self
				load: (name copyFrom: 1 to: name size - 3)]! !

!SIHaskell methodsFor: 'communicating' stamp: 'TO 9/14/2012 08:22'!
callback: anObject 
	((anObject isKindOf: SIAlgebraicData)
			and: [anObject constructor = 'SIMessageSend'])
		ifTrue: [^ anObject args first perform: anObject args second asSymbol withArguments: anObject args third asArray].
	^ nil! !

!SIHaskell methodsFor: 'processing' stamp: 'tomooda 10/30/2012 15:04'!
evaluate: aString ifError: errorBlock 
	^ super
		evaluate: aString
		ifError: [:msg | 
			| response |
			self send: ':t ' , aString.
			response := self readUpToPrompt.
			response first = Character cr
				ifTrue: [errorBlock value: msg]
				ifFalse: [SIExpressionData with: aString in: self]]! !

!SIHaskell methodsFor: 'communicating' stamp: 'tomooda 10/31/2012 11:03'!
expression: aString ifError: errorReceivingBlock 
	| response |
	response := self trimPrompt: aString.
	response
		linesDo: [:line | line isEmpty
				ifTrue: [^ errorReceivingBlock value: response]].
	^ response! !

!SIHaskell methodsFor: 'class access' stamp: 'TomohiroOda 7/24/2012 11:19'!
expressionParserClass
	^ SIHaskellValueParser! !

!SIHaskell methodsFor: 'class access' stamp: 'TO 7/27/2012 17:31'!
expressionPrinterClass
	^ SIHaskellExpressionPrinter! !

!SIHaskell methodsFor: 'private' stamp: 'TO 10/19/2012 11:30'!
forkProcess: desc 
	(OSProcess accessor isExecutable: self ghciPath) = true
		ifFalse: [| msg |
			msg := String new writeStream.
			msg nextPutAll: 'GHCi interpreter not found at ';
				 nextPutAll: self ghciPath;
				 nextPutAll: '.';
				 cr;
				 nextPutAll: 'Please specify its fullpath location in SIHaskell class>>ghciPath.';
				 cr;
				 nextPutAll: 'If you don''t have one, please visit http://www.haskell.org/ to download Haskell.'.
			SINotifier notify: msg contents.
			thisContext terminate].
	^ UnixProcess
		forkJob: self ghciPath
		arguments: #()
		environment: nil
		descriptors: desc! !

!SIHaskell methodsFor: 'accessing' stamp: 'TO 10/19/2012 11:21'!
ghciPath
	^ self class ghciPath! !

!SIHaskell methodsFor: 'initialize-release' stamp: 'TO 10/18/2012 16:30'!
initialize
	super initialize.
	programs := OrderedCollection new! !

!SIHaskell methodsFor: 'private' stamp: 'tomooda 10/30/2012 15:33'!
installUtilities
	self send: ':set prompt >'.
	reader upTo: $>.
	self send: ':add ', (FileStream fullName: 'SomethingIt.hs').
	self readUpToPrompt.
	self send: 'import SomethingIt (SIObject(SIObject), messageSend)'.
	self readUpToPrompt! !

!SIHaskell methodsFor: 'processing' stamp: 'tomooda 10/31/2012 11:06'!
load: aFileNameString 
	^ self
		load: aFileNameString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIHaskell methodsFor: 'processing' stamp: 'tomooda 10/31/2012 10:57'!
load: aFileNameString ifError: errorBlock 
	self program: ':add ' , (FileStream fullName: aFileNameString) ifError: errorBlock! !

!SIHaskell methodsFor: 'processing' stamp: 'TO 10/18/2012 16:42'!
program: aString 
	^ self
		program: aString
		ifError: [:msg | 
			SINotifier notify: msg.
			nil]! !

!SIHaskell methodsFor: 'processing' stamp: 'tomooda 10/31/2012 11:09'!
program: aString ifError: errorBlock 
	self
		critical: [process
				ifNil: [self startupProcess].
			self send: aString.
			self
				expression: self readUpToPrompt
				ifError: [:msg | ^ errorBlock value: msg].
			programs add: aString]! !

!SIHaskell methodsFor: 'private' stamp: 'tomooda 10/30/2012 15:12'!
startupProcess
	super startupProcess.
	self installUtilities.
	programs
		do: [:program | 
			| response |
			self send: program.
			response := self readUpToPrompt.
			self
				expression: response
				ifError: [self error: 'Error while reloading programs']]! !

SIExpressionPrinter subclass: #SIHaskellExpressionPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-Haskell-Process'!

!SIHaskellExpressionPrinter commentStamp: 'TO 7/25/2012 13:12' prior: 0!
A SIHaskellExpressionPrinter prints a Smalltalk object in the form of Haskell expression.

Instance Variables
	classMap:		a Dictionary that maps a Smalltalk class to a printing selector.
	escapeMap:		a Dictionary that maps a special character to an escaped string.
!

!SIHaskellExpressionPrinter methodsFor: 'defaults' stamp: 'TO 9/13/2012 12:42'!
defaultClassMap
	| map |
	map := Array new writeStream.
	map nextPut: #Array -> #printTuple:on:.
	map nextPut: #OrderedCollection -> #printList:on:.
	map nextPut: #Number -> #printNumber:on:.
	map nextPut: #String -> #printString:on:.
	map nextPut: #Character -> #printCharacter:on:.
	map nextPut: #Boolean -> #printBoolean:on:.
	map nextPut: #SIAlgebraicData -> #printData:on:.
	map nextPut: #SIExpressionData -> #printExpression:on:.
	^ map contents
! !

!SIHaskellExpressionPrinter methodsFor: 'printing' stamp: 'TomohiroOda 6/8/2012 11:19'!
printData: anObject on: aStream
	aStream nextPut: $(.
	aStream nextPutAll: anObject constructor.
	anObject args do: [ :arg |
		aStream space.
		self print: arg on: aStream].
	aStream nextPut: $)! !

!SIHaskellExpressionPrinter methodsFor: 'printing' stamp: 'TomohiroOda 9/11/2012 11:40'!
printExpression: anObject on: aStream
	aStream nextPut: $(.
	aStream nextPutAll: anObject expression.
	aStream nextPut: $)! !

!SIHaskellExpressionPrinter methodsFor: 'printing' stamp: 'TomohiroOda 9/11/2012 15:28'!
printFunc: aString arguments: anArray on: aStream
	aStream nextPut: $(.
	aStream nextPutAll: aString.
	anArray
		do: [ :object | 
			aStream nextPut: Character space.
			self print: object on: aStream ].
	aStream nextPut: $)! !

!SIHaskellExpressionPrinter methodsFor: 'printing' stamp: 'TO 9/13/2012 13:06'!
printObject: anObject on: aStream
	| oid |
	oid := interpreter activationRecord addLast: anObject; size.
	aStream nextPut: $(.
	aStream nextPutAll: 'SIObject '.
	oid printOn: aStream.
	aStream nextPut: $)! !

!String methodsFor: '*SomethingIt-Haskell-converting' stamp: 'TO 9/12/2012 13:07'!
asHaskellExpression
	^SIExpressionData with: self! !

!String methodsFor: '*SomethingIt-Haskell-converting' stamp: 'TO 10/18/2012 09:37'!
asHaskellExpressionIn: anSIHaskell 
	^ SIExpressionData with: self in: anSIHaskell! !

!String methodsFor: '*SomethingIt-Haskell-converting' stamp: 'TO 9/14/2012 09:48'!
asHaskellExpressionTyped: aTypeString 
	^ SIExpressionData with: '((' , self , ')::(' , aTypeString , '))'! !

!String methodsFor: '*SomethingIt-Haskell-converting' stamp: 'TO 10/18/2012 09:37'!
asHaskellExpressionTyped: aTypeString in: anSIHaskell 
	^ SIExpressionData with: '((' , self , ')::(' , aTypeString , '))' in: anSIHaskell! !

!String methodsFor: '*SomethingIt-Haskell-converting' stamp: 'TO 9/21/2012 07:49'!
asHaskellLazyExpression
	^ (SIExpressionData with: self) isLazy: true;
		 yourself! !

!String methodsFor: '*SomethingIt-Haskell-converting' stamp: 'TO 10/18/2012 09:38'!
asHaskellLazyExpressionIn: anSIHaskell 
	^ (SIExpressionData with: self in: anSIHaskell) isLazy: true;
		 yourself! !

!String methodsFor: '*SomethingIt-Haskell-converting' stamp: 'TO 9/21/2012 07:49'!
asHaskellLazyExpressionTyped: aTypeString 
	^ (SIExpressionData with: '((' , self , ')::(' , aTypeString , '))') isLazy: true;
		 yourself! !

!String methodsFor: '*SomethingIt-Haskell-converting' stamp: 'TO 10/18/2012 09:38'!
asHaskellLazyExpressionTyped: aTypeString in: anSIHaskell 
	^ (SIExpressionData with: '((' , self , ')::(' , aTypeString , '))' in: anSIHaskell) isLazy: true;
		 yourself! !

!SmalltalkEditor class methodsFor: '*SomethingIt-Haskell-shortcuts' stamp: 'TO 9/12/2012 15:23'!
initializeCmdKeyShortcuts
	"Initialize the (unshifted) command-key (or alt-key) shortcut table."
	"NOTE: if you don't know what your keyboard generates, use Sensor kbdTest"
	"SmalltalkEditor initialize"
	| cmds |
	super initializeCmdKeyShortcuts.
	cmds := #($b #browseIt: $d #doIt: $h #haskellIt: $i #inspectIt: $j #doAgainOnce: $l #cancel: $m #implementorsOfIt: $n #sendersOfIt: $o #spawnIt: $p #printIt: $q #querySymbol: $s #save: ).
	1 to: cmds size
		by: 2
		do: [ : i | cmdActions at: (cmds at: i) asciiValue + 1 put: (cmds at: i + 1)].
	"Set up type-method argument hot keys, 1-4.."
	'1234' do:
		[ : eachKeyboardChar |
		cmdActions 
			at: eachKeyboardChar asciiValue + 1
			put: #typeMethodArgument: ]! !

!SmalltalkEditor class methodsFor: '*SomethingIt-Haskell-shortcuts' stamp: 'TO 10/17/2012 13:17'!
initializeShiftCmdKeyShortcuts 
	"Initialize the shift-command-key (or control-key) shortcut table."
	"NOTE: if you don't know what your keyboard generates, use Sensor kbdTest"
	"wod 11/3/1998: Fix setting of cmdMap for shifted keys to actually use the 
	capitalized versions of the letters.
	TPR 2/18/99: add the plain ascii values back in for those VMs that don't return the shifted values."

	"SmalltalkEditor initialize"

	| cmds |
	super initializeShiftCmdKeyShortcuts.
	
	cmds := #(
		$a	argAdvance:
		$b	browseItHere:
		$e	methodStringsContainingIt:
		$f	displayIfFalse:
		$g	fileItIn:
		$h	inspectHaskellIt:
		$i	exploreIt:
		$n	referencesToIt:
		$s	invokePrettyPrint:
		$t	displayIfTrue:
		$v	pasteInitials:
		$w	methodNamesContainingIt:
	).
	1 to: cmds size by: 2 do: [ :i |
		shiftCmdActions at: ((cmds at: i) asciiValue + 1) put: (cmds at: i + 1).			"plain keys"
		shiftCmdActions at: ((cmds at: i) asciiValue - 32 + 1) put: (cmds at: i + 1).		"shifted keys"
		shiftCmdActions at: ((cmds at: i) asciiValue - 96 + 1) put: (cmds at: i + 1).		"ctrl keys"
	].! !

TestCase subclass: #SIHaskellParserPrinterTest
	instanceVariableNames: 'parser printer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-Haskell-Test'!

!SIHaskellParserPrinterTest methodsFor: 'running' stamp: 'TO 7/25/2012 13:09'!
setUp
	parser := SIHaskellValueParser new.
	printer := SIHaskellExpressionPrinter new! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:14'!
test: anObject
	self assert: anObject equals: (parser parse: (printer print: anObject))! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:25'!
testAlgebraicDataType
	self
		test: (SIAlgebraicData of: 'Freq' withAll: #(4 4 1 )).
	self
		test: (SIAlgebraicData of: 'Freq')! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:14'!
testBoolean
	self test: true.
	self test: false! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:51'!
testCharacter
	self test: $a.
	self test: $z! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:49'!
testInteger
	self
		assert: 441
		equals: (parser parseInteger: '441' readStream).
	self
		assert: -441
		equals: (parser parseInteger: '-441' readStream)! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:15'!
testList
	self test: #(4 4 1) asOrderedCollection.
	self test: #(4) asOrderedCollection.
	self test: #() asOrderedCollection! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:57'!
testLowerIdentifierChar
	self assert: (parser isLowerIdentifierChar: $a).
	self assert: (parser isLowerIdentifierChar: $1).
	self assert: (parser isLowerIdentifierChar: $_).
	self deny: (parser isLowerIdentifierChar: $A).
	self deny: (parser isLowerIdentifierChar: $+).
! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:50'!
testNat
	self
		assert: 441
		equals: (parser parseNaturalNumber: '441' readStream).
! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:16'!
testNumber
	self test: 1.
	self test: -2.
	self test: 3.4.
	self test: -5.6.
	self test: -7.8e-9.
! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:17'!
testParen
	self test: (SIAlgebraicData of: 'Fr' withAll: (Array with: (SIAlgebraicData of: 'Q' withAll: #(25))))
! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:51'!
testString
	self test: 'Hello world'.
	self test: 'How''ya doing?\nI''m fine, thanks!!'! !

!SIHaskellParserPrinterTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:55'!
testTuple
	self test: #(4 4 1 ) .
	self test: #('Freq' 25)! !

TestCase subclass: #SIHaskellTest
	instanceVariableNames: 'haskell'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-Haskell-Test'!

!SIHaskellTest methodsFor: 'running' stamp: 'TO 7/25/2012 13:30'!
setUp
	haskell := SIHaskell prelude! !

!SIHaskellTest methodsFor: 'running' stamp: 'TO 7/25/2012 13:30'!
tearDown
	haskell close.
	haskell := nil! !

!SIHaskellTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:35'!
testArithmetic
	self
		assert: 7
		equals: (haskell evaluate: '1+2*3')! !

!SIHaskellTest methodsFor: 'testing' stamp: 'tomooda 11/1/2012 12:18'!
testCallback
	haskell program: 'let f x = messageSend x "*" [2]::IO Int'.
	self
		assert: 6
		equals: (haskell evaluate: 'f 3')! !

!SIHaskellTest methodsFor: 'testing' stamp: 'tomooda 11/1/2012 12:20'!
testCallbackInModule
	haskell := SIHaskell source: '
import SomethingIt (messageSend)
f :: Int -> IO Int
f x = messageSend x "*" [2]'.
	self
		assert: 6
		equals: (haskell evaluate: 'f 3')! !

!SIHaskellTest methodsFor: 'testing' stamp: 'TO 7/26/2012 13:40'!
testLoad
	SIFileUtils
		temporaryFile: 'SIHaskellTest-testLoad-'
		randomSuffixSize: 8
		suffix: '.hs'
		do: [:filename :stream | 
			stream nextPutAll: 'fizzbuzz x = if fz == "" then (show x) else fz where fz = fizz x ++ buzz x';
				 lf.
			stream nextPutAll: 'fizz x = if (mod x 3) == 0 then "fizz" else ""';
				 lf.
			stream nextPutAll: 'buzz x = if (mod x 5) == 0 then "buzz" else ""';
				 lf.
			stream close.
			haskell close.
			haskell := SIHaskell
				load: (filename copyFrom: 1 to: filename size - 3).
			self
				assert: '13'
				equals: (haskell evaluate: 'fizzbuzz 13').
			self
				assert: 'fizz'
				equals: (haskell evaluate: 'fizzbuzz 6').
			self
				assert: 'buzz'
				equals: (haskell evaluate: 'fizzbuzz 20').
			self
				assert: 'fizzbuzz'
				equals: (haskell evaluate: 'fizzbuzz 30').
			haskell close]! !

!SIHaskellTest methodsFor: 'testing' stamp: 'TO 7/25/2012 13:37'!
testMultiArithmetic
	self
		assert: 7
		equals: (haskell evaluate: '1+2*3').
	self
		assert: 34
		equals: (haskell evaluate: '4+5*6')! !

!SIHaskellTest methodsFor: 'testing' stamp: 'TO 10/18/2012 09:00'!
testObject
	| object |
	haskell program: 'let enlist x = [x]'.
	object := Object new.
	self assert: object equals: ((haskell evaluate: 'enlist')
			value: object) first! !

!SIHaskellTest methodsFor: 'testing' stamp: 'TO 7/25/2012 14:28'!
testProgram
	haskell program: 'let f x = 2 * x + 3'.
	self
		assert: 11
		equals: (haskell evaluate: 'f 4').
	haskell program: 'let f x = 5 * x + 6'.
	self
		assert: 41
		equals: (haskell evaluate: 'f 7')! !

!SIHaskellTest methodsFor: 'testing' stamp: 'tomooda 11/1/2012 12:16'!
testSource
	self
		assert: 6
		equals: ((SIHaskell source: 'f x = x * 2')
				evaluate: 'f(3)')! !

!TextEditor methodsFor: '*SomethingIt-Haskell-menu messages' stamp: 'tomooda 11/1/2012 12:46'!
haskellIt
	^ self
		haskellSelectionAndDo: [:result | self afterSelectionInsertAndSelect: result]! !

!TextEditor methodsFor: '*SomethingIt-Haskell-menu messages' stamp: 'TO 9/12/2012 15:16'!
haskellIt: event 
	self haskellIt.
	^ true! !

!TextEditor methodsFor: '*SomethingIt-Haskell-menu messages' stamp: 'tomooda 11/1/2012 12:46'!
haskellSelectionAndDo: aBlock 
	self
		lineSelectAndEmptyCheck: [^ nil].
	^ (SIHaskell prelude plainEvaluate: self selection asString limit: 500)
		ifNotNilDo: aBlock! !

!TextEditor methodsFor: '*SomethingIt-Haskell-menu messages' stamp: 'TO 10/17/2012 13:18'!
inspectHaskellIt
	self
		lineSelectAndEmptyCheck: [^ nil].
	(SIHaskell prelude evaluate: self selection asString)
		ifNotNilDo: [:result | result inspect]! !

!TextEditor methodsFor: '*SomethingIt-Haskell-menu messages' stamp: 'TO 10/17/2012 13:17'!
inspectHaskellIt: event 
	self inspectHaskellIt.
	^ true! !

!StringHolder class methodsFor: '*SomethingIt-Haskell-menu' stamp: 'TO 10/17/2012 13:15'!
yellowButtonMenuItems
	"Returns the standard yellow button menu items"
	^{
			{'set font... (k)' translated.				#offerFontMenu}.
			{'set style... (K)' translated.				#changeStyle}.
			{'set alignment... (u)' translated.		#chooseAlignment}.
			#-.
			{'make project link (P)' translated.	#makeProjectLink}.
			#-.
			{'find...(f)' translated.					#find}.
			{'find again (g)' translated.				#findAgain}.
			{'set search string (h)' translated.		#setSearchString}.
			#-.
			{'do again (j)' translated.				#again}.
			{'undo (z)' translated.					#undo}.
			#-.
			{'copy (c)' translated.					#copySelection}.
			{'cut (x)' translated.						#cut}.
			{'paste (v)' translated.					#paste}.
			{'paste...' translated.					#pasteRecent}.
			#-.
			{'do it (d)' translated.		#doIt}.			     
		{'print it (p)' translated.		#printIt}.		     
		{'inspect it (i)' translated.		#inspectIt}.		     
		{'explore it (I)' translated.		#exploreIt}.		     
		{'debug it' translated.			#debugIt}.		     
		{'button for it' translated.			#buttonForIt}.		     
		{'tally it' translated.			#tallyIt}.
			#-.
			{'haskell it (h)' translated.	#printHaskellIt}.
			{'inspect haskell it (H)' translated. #inspectHaskellIt}.
			#-.
			{'accept (s)' translated.					#accept}.
			{'cancel (l)' translated.					#cancel}.
			#-.
			{'show bytecodes' translated.			#showBytecodes}.
			#-.
			{'copy html' translated.					#copyHtml}.
			#-.
			{'more...' translated.					#shiftedTextPaneMenuRequest}.
		}! !

SIValueParser subclass: #SIHaskellValueParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-Haskell-Process'!

!SIHaskellValueParser commentStamp: 'TO 7/25/2012 11:36' prior: 0!
A SIHaskellValueParser parses an expression from GHCi. The parser accepts only a canonical normal form, neither a Haskell program nor an expression that can be reduced.

Instance Variables
!

!SIHaskellValueParser methodsFor: 'defaults' stamp: 'tomooda 11/1/2012 10:51'!
defaultClassMap
	^ super defaultClassMap
		at: #Data
		put: [:constructor :args | self constructAlgebraicData: constructor args: args];
		 yourself! !

!SIHaskellValueParser methodsFor: 'parsing' stamp: 'TO 9/13/2012 13:18'!
parseAlgebraicData: aReadStream 
	| identifier args |
	identifier := self parseIdentifier: aReadStream.
	identifier = 'True'
		ifTrue: [^ true].
	identifier = 'False'
		ifTrue: [^ false].
	identifier = 'SIObject'
		ifTrue: [aReadStream next.
			^ interpreter activationRecord
				at: (self parseNaturalNumber: aReadStream)].
	(aReadStream atEnd not
			and: [aReadStream peek = Character space])
		ifFalse: [^ (classMap at: #Data)
				value: identifier
				value: Array new].
	aReadStream next.
	aReadStream peek = ${
		ifTrue: [| bindings |
			bindings := Dictionary new.
			aReadStream next.
			[aReadStream atEnd not
				and: [aReadStream peek ~= $}]]
				whileTrue: [| key value |
					key := self parseIdentifier: aReadStream.
					aReadStream next = Character space
						ifFalse: [^ self error: 'Invalid record format. Missing space after name.'].
					aReadStream next = $=
						ifFalse: [^ self error: 'Invalid record format. Missing =.'].
					aReadStream next = Character space
						ifFalse: [^ self error: 'Invalid record format Missing space after ='].
					value := self parseStream: aReadStream.
					bindings at: key asSymbol put: value.
					aReadStream peek = $,
						ifTrue: [aReadStream next.
							aReadStream next = Character space
								ifFalse: [^ self error: 'Missing space after comma.']]
						ifFalse: [aReadStream peek = $}
								ifFalse: [^ self error: 'Missing } in record expression']]].
			^ (classMap at: #Record)
				value: bindings]
		ifFalse: [args := Array new writeStream.
			args
				nextPut: (self parseStream: aReadStream).
			[aReadStream atEnd not
				and: [aReadStream peek = Character space]]
				whileTrue: [aReadStream next.
					args
						nextPut: (self parseStream: aReadStream)].
			^ (classMap at: #Data)
				value: identifier
				value: args contents]! !

!SIHaskellValueParser methodsFor: 'parsing' stamp: 'TomohiroOda 9/11/2012 09:54'!
parseErrorMessage: aStream
	| message |
	message := String new writeStream.
	[ 
	| char |
	aStream atEnd
		ifTrue: [ ^ message contents ].
	char := aStream next.
	char = Character lf
		ifTrue: [ char := Character cr ].
	message nextPut: char.
	(char = Character cr and: [ aStream peek ~= Character space ])
		ifTrue: [ ^ message contents ] ] repeat! !

!SIHaskellValueParser methodsFor: 'parsing' stamp: 'TO 7/25/2012 11:51'!
parseParen: aReadStream
	| expressions |
	aReadStream next = $(
		ifFalse: [ ^ self error: 'Invalid expression. Missing $(' ].
	expressions := self parseExpressionList: aReadStream separatedBy: ','.
	aReadStream next = $)
		ifFalse: [ ^ self error: 'Invalid expression. Missing $)' ].
	expressions isEmpty
		ifTrue: [ ^ self error: 'Invalid expression. Empty expression.' ].
	expressions size = 1
		ifTrue: [ ^ expressions first ].
	^ (classMap at: #Tuple) value: expressions contents! !

!SIHaskellValueParser methodsFor: 'parsing' stamp: 'TomohiroOda 9/10/2012 15:38'!
parseStream: aReadStream ifError: errorBlock
	| char |
	self skipSeparator: aReadStream.
	char := aReadStream peek.
	(char isDigit or: [char = $-])
		ifTrue: [ ^ self parseNumber: aReadStream ].
	(self isUpperIdentifierChar: char)
		ifTrue: [ ^ self parseAlgebraicData: aReadStream ].
	char = $(
		ifTrue: [ ^ self parseParen: aReadStream ].
	char = $[
		ifTrue: [ ^ self parseList: aReadStream ].
	char = $"
		ifTrue: [ ^ self parseString: aReadStream ].
	char = $'
		ifTrue: [ ^ self parseCharacter: aReadStream ].
	^ errorBlock value: (self parseErrorMessage: aReadStream)! !

!PluggableTextMorph methodsFor: '*SomethingIt-Haskell-menu commands' stamp: 'TO 10/17/2012 13:16'!
inspectHaskellIt
	textMorph editor inspectHaskellIt! !

!PluggableTextMorph methodsFor: '*SomethingIt-Haskell-menu commands' stamp: 'tomooda 11/2/2012 10:26'!
printHaskellIt
	| oldEditor |
	textMorph editor selectFrom: selectionInterval first to: selectionInterval last;
						model: model.
	textMorph handleEdit: [(oldEditor := textMorph editor) haskellSelectionAndDo:
		[:result |
		selectionInterval := oldEditor selectionInterval.
		textMorph installEditorToReplace: oldEditor.
		textMorph handleEdit: [oldEditor afterSelectionInsertAndSelect: result].
		selectionInterval := oldEditor selectionInterval.
	
		textMorph editor selectFrom: selectionInterval first to: selectionInterval last.
		self scrollSelectionIntoView]]! !
SIHaskell initialize!
